@@ -1,6 +1,3 @@
-/* $Xorg: xinit.c,v 1.5 2001/02/09 02:05:49 xorgcvs Exp $ */
-/* $XdotOrg: $ */
-
 /*
 
 Copyright 1986, 1998  The Open Group
@@ -26,7 +23,6 @@ used in advertising or otherwise to promote the sale, use or other dealings
 in this Software without prior written authorization from The Open Group.
 
 */
-/* $XFree86: xc/programs/xinit/xinit.c,v 3.32 2002/05/31 18:46:13 dawes Exp $ */
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
@@ -39,12 +35,6 @@ in this Software without prior written authorization from The Open Group.
 #include <ctype.h>
 #include <stdint.h>
 
-#ifdef __APPLE__
-#include <CoreServices/CoreServices.h>
-#define kX11AppBundleId "org.x.X11"
-#define kX11AppBundlePath "/Contents/MacOS/X11"
-#endif
-
 #ifdef X_POSIX_C_SOURCE
 #define _POSIX_C_SOURCE X_POSIX_C_SOURCE
 #include <signal.h>
@@ -66,6 +56,13 @@ in this Software without prior written authorization from The Open Group.
 #include <setjmp.h>
 #include <stdarg.h>
 
+#ifdef __APPLE__
+#include <AvailabilityMacros.h>
+#if MAC_OS_X_VERSION_MIN_REQUIRED >= 1060
+#include <vproc.h>
+#endif
+#endif
+
 #if !defined(SIGCHLD) && defined(SIGCLD)
 #define SIGCHLD SIGCLD
 #endif
@@ -101,12 +98,6 @@ char **newenvironlast = NULL;
 # endif
 #endif
 
-/* A/UX setpgid incorrectly removes the controlling terminal.
-   Per Posix, only setsid should do that. */
-#ifdef macII
-#define setpgid setpgrp
-#endif
-
 #ifdef __UNIXOS2__
 #define HAS_EXECVPE
 #endif
@@ -119,36 +110,21 @@ char **newenvironlast = NULL;
 
 const char *bindir = BINDIR;
 const char * const server_names[] = {
-#if defined(ultrix) && defined(mips)
-    "Xdec        Digital color display on DECstation",
-#endif
-#if defined(sun) && !defined(XORG)	/* Sun */
-    "Xsun        Sun BW2, CG2, CG3, CG4, or CG6 on Sun 2, 3, 4, or 386i",
-    "Xsunmono    Sun BW2 on Sun 2, 3, 4, or 386i ",
-    "Xsun24      Sun BW2, CG2, CG3, CG4, CG6, or CG8 on Sun 4",
-#endif
-#ifdef hpux				/* HP */
-    "Xhp         HP monochrome and colors displays on 9000/300 series",
-#endif
-#ifdef ibm				/* IBM */
-    "Xibm        IBM AED, APA, 8514a, megapel, VGA displays on PC/RT",
-#endif
-#ifdef macII				/* MacII */
-    "XmacII      Apple monochrome display on Macintosh II",
-#endif
-#ifdef XFREE86
-    "XFree86     XFree86 displays",
-#endif
-#ifdef XORG
-    "Xorg        Common X server for most displays",
-#endif
 #ifdef __APPLE__
     "Xquartz     Mac OSX Quartz displays.",
+#else
+# ifdef __CYGWIN__
+    "XWin        X Server for the Cygwin environment on Microsoft Windows",
+# else
+    "Xorg        Common X server for most displays",
+# endif
 #endif
     "Xvfb        Virtual frame buffer",
     "Xfake       kdrive-based virtual frame buffer",
     "Xnest       X server nested in a window on another X server",
     "Xephyr      kdrive-based nested X server",
+    "Xvnc        X server accessed over VNC's RFB protocol",
+    "Xdmx        Distributed Multi-head X server",
     NULL};
 
 #ifndef XINITRC
@@ -176,9 +152,6 @@ static char **client = clientargv + 2;		/* make sure room for sh .xinitrc args *
 static char *displayNum = NULL;
 static char *program = NULL;
 static Display *xd = NULL;			/* server connection */
-#ifdef __APPLE__
-static char x11_path[PATH_MAX];
-#endif
 #ifndef SYSV
 #if defined(__CYGWIN__) || defined(SVR4) || defined(_POSIX_SOURCE) || defined(CSRG_BASED) || defined(__UNIXOS2__) || defined(Lynx) || defined(__APPLE__)
 int status;
@@ -228,56 +201,11 @@ sigUsr1(int sig)
 #endif
 }
 
-#ifdef __APPLE__
-static void set_x11_path() {
-    CFURLRef appURL = NULL;
-    OSStatus osstatus = LSFindApplicationForInfo(kLSUnknownCreator, CFSTR(kX11AppBundleId), nil, nil, &appURL);
-
-    switch (osstatus) {
-        case noErr:
-            if (appURL == NULL) {
-                fprintf(stderr, "xinit: Invalid response from LSFindApplicationForInfo(%s)\n", 
-                        kX11AppBundleId);
-                exit(1);
-            }
-            
-            if (!CFURLGetFileSystemRepresentation(appURL, true, (unsigned char *)x11_path, sizeof(x11_path))) {
-                fprintf(stderr, "xinit: Error resolving URL for %s\n", kX11AppBundleId);
-                exit(2);
-            }
-            
-            strlcat(x11_path, kX11AppBundlePath, sizeof(x11_path));
-#ifdef DEBUG
-            fprintf(stderr, "xinit: X11.app = %s\n", x11_path);
-#endif
-            break;
-        case kLSApplicationNotFoundErr:
-            fprintf(stderr, "xinit: Unable to find application for %s\n", kX11AppBundleId);
-            exit(4);
-        default:
-            fprintf(stderr, "xinit: Unable to find application for %s, error code = %d\n", 
-                    kX11AppBundleId, (int)osstatus);
-            exit(5);
-    }
-}
-#endif
-
 static void 
 Execute(char **vec,		/* has room from up above */
 	char **envp)
 {
-    char *file = vec[0];
-#ifdef __APPLE__
-    /* This is ugly, but currently, we need to trick OS-X into thinking X is in
-     * the X11.app bundle.  Hopefully UI, icons, etc will eventually be set
-     * by Xquartz, but this is how we're doing it for now. -JH
-     */
-    if(!strcmp(file, "/usr/X11/bin/X") || !strcmp(file, "/usr/X11/bin/Xquartz") || !strcmp(file, "X") || !strcmp(file, "Xquartz")) {
-        vec[0] = x11_path;
-        fprintf(stderr, "xinit: Detected Xquartz startup, setting file=%s, argv[0]=%s\n", file, vec[0]);
-    }
-#endif
-    Execvpe (file, vec, envp);
+    Execvpe (vec[0], vec, envp);
 #ifndef __UNIXOS2__
     if (access (vec[0], R_OK) == 0) {
 	vec--;				/* back it up to stuff shell in */
@@ -304,9 +232,10 @@ main(int argc, char *argv[], char *envp[])
 	int client_args_given = 0, server_args_given = 0;
 	int start_of_client_args, start_of_server_args;
 	struct sigaction sa;
-
 #ifdef __APPLE__
-    set_x11_path();
+#if MAC_OS_X_VERSION_MIN_REQUIRED >= 1060
+	vproc_transaction_t vt;
+#endif
 #endif
 
 #ifdef __UNIXOS2__
@@ -344,15 +273,6 @@ main(int argc, char *argv[], char *envp[])
 #endif
 		for (ptr = default_client; *ptr; )
 			*cptr++ = *ptr++;
-#ifdef sun
-		/* 
-		 * If running on a sun, and if WINDOW_PARENT isn't defined, 
-		 * that means SunWindows isn't running, so we should pass 
-		 * the -C flag to xterm so that it sets up a console.
-		 */
-		if ( getenv("WINDOW_PARENT") == NULL )
-		    *cptr++ = "-C";
-#endif /* sun */
 	} else {
 		client_given = 1;
 	}
@@ -412,10 +332,11 @@ main(int argc, char *argv[], char *envp[])
 
 	    xinitrcbuf[0] = '\0';
 	    if ((cp = getenv ("XINITRC")) != NULL) {
-		strcpy (xinitrcbuf, cp);
+		(void) snprintf (xinitrcbuf, sizeof(xinitrcbuf), "%s", cp);
 		required = True;
 	    } else if ((cp = getenv ("HOME")) != NULL) {
-		(void) sprintf (xinitrcbuf, "%s/%s", cp, XINITRC);
+		(void) snprintf (xinitrcbuf, sizeof(xinitrcbuf),
+				 "%s/%s", cp, XINITRC);
 	    }
 	    if (xinitrcbuf[0]) {
 		if (access (xinitrcbuf, F_OK) == 0) {
@@ -439,10 +360,11 @@ main(int argc, char *argv[], char *envp[])
 
 	    xserverrcbuf[0] = '\0';
 	    if ((cp = getenv ("XSERVERRC")) != NULL) {
-		strcpy (xserverrcbuf, cp);
+		(void) snprintf (xserverrcbuf, sizeof(xserverrcbuf), "%s", cp);
 		required = True;
 	    } else if ((cp = getenv ("HOME")) != NULL) {
-		(void) sprintf (xserverrcbuf, "%s/%s", cp, XSERVERRC);
+		(void) snprintf (xserverrcbuf, sizeof(xserverrcbuf),
+				 "%s/%s", cp, XSERVERRC);
 	    }
 	    if (xserverrcbuf[0]) {
 		if (access (xserverrcbuf, F_OK) == 0) {
@@ -482,6 +404,13 @@ main(int argc, char *argv[], char *envp[])
 
 	signal(SIGALRM, sigAlarm);
 	signal(SIGUSR1, sigUsr1);
+
+#ifdef __APPLE__
+#if MAC_OS_X_VERSION_MIN_REQUIRED >= 1060
+	vt = vproc_transaction_begin(NULL);
+#endif
+#endif
+
 	if (startServer(server) > 0
 	 && startClient(client) > 0) {
 		pid = -1;
@@ -490,6 +419,13 @@ main(int argc, char *argv[], char *envp[])
 			)
 			pid = wait(NULL);
 	}
+
+#ifdef __APPLE__
+#if MAC_OS_X_VERSION_MIN_REQUIRED >= 1060
+	vproc_transaction_end(NULL, vt);
+#endif
+#endif
+
 	signal(SIGTERM, SIG_IGN);
 	signal(SIGQUIT, SIG_IGN);
 	signal(SIGINT, SIG_IGN);
@@ -520,15 +456,23 @@ waitforserver(void)
 	int	ncycles	 = 120;		/* # of cycles to wait */
 	int	cycles;			/* Wait cycle count */
 
+#ifdef __APPLE__
+	/* For Apple, we don't get signaled by the server when it's ready, so we just
+	 * want to sleep now since we're going to sleep later anyways and this allows us
+	 * to avoid the awkard, "why is there an error message in the log" questions
+	 * from users.
+         */
+
+	sleep(2);
+#endif
+
 	for (cycles = 0; cycles < ncycles; cycles++) {
 		if ((xd = XOpenDisplay(displayNum))) {
 			return(TRUE);
 		}
 		else {
-#define MSG "X server to begin accepting connections"
-		    if (!processTimeout (1, MSG)) 
+		    if (!processTimeout (1, "X server to begin accepting connections")) 
 		      break;
-#undef MSG
 		}
 	}
 
@@ -705,6 +649,7 @@ setWindowPath(void)
 	unsigned long num;
 	char nums[10];
 	int numn;
+	size_t len;
 	prop = XInternAtom(xd, "XFree86_VT", False);
 	if (prop == None) {
 #ifdef DEBUG
@@ -760,11 +705,18 @@ setWindowPath(void)
 	windowpath = getenv("WINDOWPATH");
 	numn = snprintf(nums, sizeof(nums), "%lu", num);
 	if (!windowpath) {
-		newwindowpath = malloc(10 + 1 + numn + 1);
-		sprintf(newwindowpath, "WINDOWPATH=%s", nums);
+		len = 10 + 1 + numn + 1;
+		newwindowpath = malloc(len);
+		if (newwindowpath == NULL) 
+		    return;
+		snprintf(newwindowpath, len, "WINDOWPATH=%s", nums);
 	} else {
-		newwindowpath = malloc(10 + 1 + strlen(windowpath) + 1 + numn + 1);
-		sprintf(newwindowpath, "WINDOWPATH=%s:%s", windowpath, nums);
+		len = 10 + 1 + strlen(windowpath) + 1 + numn + 1;
+		newwindowpath = malloc(len);
+		if (newwindowpath == NULL)
+		    return;
+		snprintf(newwindowpath, len, "WINDOWPATH=%s:%s", 
+			 windowpath, nums);
 	}
 	*newenvironlast++ = newwindowpath;
 	*newenvironlast = NULL;
@@ -887,8 +839,7 @@ set_environment(void)
     }
 
     /* put DISPLAY=displayname as first element */
-    strcpy (displaybuf, "DISPLAY=");
-    strcpy (displaybuf + 8, displayNum);
+    snprintf (displaybuf, sizeof(displaybuf), "DISPLAY=%s", displayNum);
     newPtr = newenviron;
     *newPtr++ = displaybuf;
 
